@@ -23,6 +23,8 @@ from lambda_max import lambda_max
 from lambda_min import lambda_min
 from log_det import log_det
 from log_sum_exp import log_sum_exp
+from max_entries import max_entries
+from min_entries import min_entries
 from norm import norm
 from norm1 import norm1
 from norm2 import norm2
@@ -32,9 +34,12 @@ from quad_form import quad_form
 from quad_over_lin import quad_over_lin
 from sigma_max import sigma_max
 from sum_squares import sum_squares
+from total_variation import tv
 
 from affine.conv import conv
 from affine.hstack import hstack
+from affine.mul_elemwise import mul_elemwise
+from affine.reshape import reshape
 from affine.sum_entries import sum_entries
 from affine.vstack import vstack
 
