@@ -19,6 +19,7 @@ along with CVXPY.  If not, see <http://www.gnu.org/licenses/>.
 
 from cvxpy.atoms import *
 from cvxpy.expressions.variables import Variable
+from cvxpy.expressions.constants import Parameter
 import cvxpy.utilities as u
 import cvxpy.interface.matrix_utilities as intf
 import numpy as np
@@ -75,8 +76,8 @@ class TestAtoms(unittest.TestCase):
         self.assertEquals(norm2(atom).curvature, u.Curvature.CONVEX_KEY)
         self.assertEquals(norm2(-atom).curvature, u.Curvature.CONVEX_KEY)
 
-    # Test quad_over_lin DCP.
     def test_quad_over_lin(self):
+        # Test quad_over_lin DCP.
         atom = quad_over_lin(square(self.x), self.a)
         self.assertEquals(atom.curvature, u.Curvature.CONVEX_KEY)
         atom = quad_over_lin(-square(self.x), self.a)
@@ -85,14 +86,45 @@ class TestAtoms(unittest.TestCase):
         self.assertEquals(atom.curvature, u.Curvature.UNKNOWN_KEY)
         assert not atom.is_dcp()
 
+        # Test quad_over_lin size validation.
+        with self.assertRaises(Exception) as cm:
+            quad_over_lin(self.x, self.x)
+        self.assertEqual(str(cm.exception),
+            "The second argument to quad_over_lin must be a scalar")
+
+    def test_elemwise_arg_count(self):
+        """Test arg count for max and min variants.
+        """
+        with self.assertRaises(Exception) as cm:
+            max_elemwise(1)
+        self.assertEqual(str(cm.exception),
+            "__init__() takes at least 3 arguments (2 given)")
+
+        with self.assertRaises(Exception) as cm:
+            min_elemwise(1)
+        self.assertEqual(str(cm.exception),
+            "__init__() takes at least 3 arguments (2 given)")
+
+    def test_max_entries_sign(self):
+        """Test sign for max_entries.
+        """
+        # One arg.
+        self.assertEquals(max_entries(1).sign, u.Sign.POSITIVE_KEY)
+        self.assertEquals(max_entries(-2).sign, u.Sign.NEGATIVE_KEY)
+        self.assertEquals(max_entries(Variable()).sign, u.Sign.UNKNOWN_KEY)
+        self.assertEquals(max_entries(0).sign, u.Sign.ZERO_KEY)
+
+    def test_min_entries_sign(self):
+        """Test sign for min_entries.
+        """
+        # One arg.
+        self.assertEquals(min_entries(1).sign, u.Sign.POSITIVE_KEY)
+        self.assertEquals(min_entries(-2).sign, u.Sign.NEGATIVE_KEY)
+        self.assertEquals(min_entries(Variable()).sign, u.Sign.UNKNOWN_KEY)
+        self.assertEquals(min_entries(0).sign, u.Sign.ZERO_KEY)
+
     # Test sign logic for max_elemwise.
     def test_max_elemwise_sign(self):
-        # One arg.
-        self.assertEquals(max_elemwise(1).sign, u.Sign.POSITIVE_KEY)
-        self.assertEquals(max_elemwise(-2).sign, u.Sign.NEGATIVE_KEY)
-        self.assertEquals(max_elemwise(Variable()).sign, u.Sign.UNKNOWN_KEY)
-        self.assertEquals(max_elemwise(0).sign, u.Sign.ZERO_KEY)
-
         # Two args.
         self.assertEquals(max_elemwise(1, 2).sign, u.Sign.POSITIVE_KEY)
         self.assertEquals(max_elemwise(1, Variable()).sign, u.Sign.POSITIVE_KEY)
@@ -112,14 +144,14 @@ class TestAtoms(unittest.TestCase):
         self.assertEquals(max_elemwise(-2, Variable(), 0, -1, Variable(), 1).sign,
                           u.Sign.POSITIVE_KEY)
 
+        # Promotion.
+        self.assertEquals(max_elemwise(1, Variable(2)).sign,
+                          u.Sign.POSITIVE_KEY)
+        self.assertEquals(max_elemwise(1, Variable(2)).size,
+                          (2, 1))
+
     # Test sign logic for min_elemwise.
     def test_min_elemwise_sign(self):
-        # One arg.
-        self.assertEquals(min_elemwise(1).sign, u.Sign.POSITIVE_KEY)
-        self.assertEquals(min_elemwise(-2).sign, u.Sign.NEGATIVE_KEY)
-        self.assertEquals(min_elemwise(Variable()).sign, u.Sign.UNKNOWN_KEY)
-        self.assertEquals(min_elemwise(0).sign, u.Sign.ZERO_KEY)
-
         # Two args.
         self.assertEquals(min_elemwise(1, 2).sign, u.Sign.POSITIVE_KEY)
         self.assertEquals(min_elemwise(1, Variable()).sign, u.Sign.UNKNOWN_KEY)
@@ -139,6 +171,12 @@ class TestAtoms(unittest.TestCase):
         self.assertEquals(min_elemwise(-2, Variable(), 0, -1, Variable(), 1).sign,
                           u.Sign.NEGATIVE_KEY)
 
+        # Promotion.
+        self.assertEquals(min_elemwise(-1, Variable(2)).sign,
+                          u.Sign.NEGATIVE_KEY)
+        self.assertEquals(min_elemwise(-1, Variable(2)).size,
+                          (2, 1))
+
     def test_sum_entries(self):
         """Test the sum_entries atom.
         """
@@ -152,6 +190,30 @@ class TestAtoms(unittest.TestCase):
         mat = np.mat("1 -1")
         self.assertEquals(sum_entries(mat*square(Variable(2))).curvature, u.Curvature.UNKNOWN_KEY)
 
+
+    def test_mul_elemwise(self):
+        """Test the mul_elemwise atom.
+        """
+        self.assertEquals(mul_elemwise([1, -1], self.x).sign, u.Sign.UNKNOWN_KEY)
+        self.assertEquals(mul_elemwise([1, -1], self.x).curvature, u.Curvature.AFFINE_KEY)
+        self.assertEquals(mul_elemwise([1, -1], self.x).size, (2, 1))
+        pos_param = Parameter(2, sign="positive")
+        neg_param = Parameter(2, sign="negative")
+        self.assertEquals(mul_elemwise(pos_param, pos_param).sign, u.Sign.POSITIVE_KEY)
+        self.assertEquals(mul_elemwise(pos_param, neg_param).sign, u.Sign.NEGATIVE_KEY)
+        self.assertEquals(mul_elemwise(neg_param, neg_param).sign, u.Sign.POSITIVE_KEY)
+
+        self.assertEquals(mul_elemwise(neg_param, square(self.x)).curvature, u.Curvature.CONCAVE_KEY)
+
+        # Test promotion.
+        self.assertEquals(mul_elemwise([1, -1], 1).size, (2, 1))
+        self.assertEquals(mul_elemwise(1, self.C).size, self.C.size)
+
+        with self.assertRaises(Exception) as cm:
+            mul_elemwise(self.x, [1, -1])
+        self.assertEqual(str(cm.exception),
+            "The first argument to mul_elemwise must be constant.")
+
     # Test the vstack class.
     def test_vstack(self):
         atom = vstack(self.x, self.y, self.x)
@@ -177,5 +239,25 @@ class TestAtoms(unittest.TestCase):
         with self.assertRaises(Exception) as cm:
             vstack()
         self.assertEqual(str(cm.exception),
-            "No arguments given to 'vstack'.")
+            "No arguments given to vstack.")
 
+    def test_reshape(self):
+        """Test the reshape class.
+        """
+        expr = reshape(self.A, 4, 1)
+        self.assertEquals(expr.sign, u.Sign.UNKNOWN_KEY)
+        self.assertEquals(expr.curvature, u.Curvature.AFFINE_KEY)
+        self.assertEquals(expr.size, (4, 1))
+
+        expr = reshape(expr, 2, 2)
+        self.assertEquals(expr.size, (2, 2))
+
+        expr = reshape(square(self.x), 1, 2)
+        self.assertEquals(expr.sign, u.Sign.POSITIVE_KEY)
+        self.assertEquals(expr.curvature, u.Curvature.CONVEX_KEY)
+        self.assertEquals(expr.size, (1, 2))
+
+        with self.assertRaises(Exception) as cm:
+            reshape(self.C, 5, 4)
+        self.assertEqual(str(cm.exception),
+            "Invalid reshape dimensions (5, 4).")
